from soccer_sdk_utils.gender import Gender
from dataclasses import dataclass


@dataclass
class Event:
    id: int
    name: str
    gender: Gender

    def __init__(self, **kwargs):
        self.id = kwargs.get("id", -1)
        self.name = kwargs.get("name", "unknown")
        self.gender = kwargs.get("gender", Gender.All)

    def __str__(self):
        return f"{self.id} - '{self.name}'"

    def __repr__(self):
        return f"Event(id={self.id}, name={self.name}, gender='{self.gender.name}')"

    def __eq__(self, other):
        return self.id == other.id

    def __lt__(self, other):
        return self.name < other.name


@dataclass
class Flight:
    division_id: int
    id: int
    name: str
    teams_count: int
    has_active_schedule: bool
    hide_settings: int

    def __init__(self, **kwargs):
        self.division_id = kwargs.get("division_id", -1)
        self.id = kwargs.get("id", -1)
        self.name = kwargs.get("name", "unknown")
        self.teams_count = kwargs.get("teams_count", 0)
        self.has_active_schedule = kwargs.get("has_active_schedule", False)
        self.hide_settings = kwargs.get("hide_settings", 0)

    def __str__(self):
        return self.name

    def __repr__(self):
        attribute_value_pairs = [
            f"division_id={self.division_id}",
            f"id={self.id}",
            f"name={self.name}",
            f"teams_count={self.teams_count}",
            f"has_active_schedule={self.has_active_schedule}",
            f"hide_settings={self.hide_settings}"
        ]

        return f"Flight({', '.join(attribute_value_pairs)})"

    def __eq__(self, other):
        return self.id == other.id

    def __lt__(self, other):
        return self.name < other.name


@dataclass
class Division:
    id: int
    name: str
    flights: list[Flight]

    def __init__(self, **kwargs):
        self.id = kwargs.get("id", -1)
        self.name = kwargs.get("name", "unknown")
        self.flights = kwargs.get("flights", [])

    def __str__(self):
        return self.name

    def __repr__(self):
        return f"Division(id={self.id}, name={self.name})"

    def __eq__(self, other):
        return self.id == other.id

    def __lt__(self, other):
        return self.name < other.name



from soccer_sdk_utils.model.meta import MetaProperty


class Team:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.title = kwargs.get("title")
        self.abbreviation = kwargs.get("abbreviation")

        self.meta = kwargs.get("meta", [])

    def __repr__(self):
        avps = []

        if self.id is not None:
            avps.append(f"id='{self.id}'")

        avps.append(f"title='{self.title}'")


        return f"<Team({', '.join(avps)})>"

    def has_property(self, name: str) -> bool:
        for item in self.meta:
            if item.name == name:
                return item.value is not None and item.value != ""

        return False

    def get_property(self, name: str) -> str | None:
        for item in self.meta:
            if item.name == name:
                return item.value

        return None

    def update_property(self, name: str, value: str | None):
        for item in self.meta:
            if item.name == name:
                item.value = value

    def add_property(self, name: str, value: str | None):
        if value is not None:

            if type(value) == str:
                value = value.strip()

                if len(value) == 0:
                    value = None

        if self.has_property(name):
            self.update_property(name, value)
        else:
            self.meta.append(MetaProperty(name=name, value=value))



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class ConferenceDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("conference", "/id", **kwargs)

    def exists(self, division: str, name: str):
        conference = self.get_by_division_and_name(division, name)

        if conference is None:
            return False

        return True

    def get_by_division_and_name(self, division: str, name: str):
        try:
            query = f"{self._select_prefix()} c WHERE c.division = '{division}' AND c.name = '{name}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            conferences = list(items)
            if len(conferences) == 0:
                return None
            else:
                return conferences[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.images import Images


class ImagesSchema(Schema):
    logo = fields.Str(allow_none=True, load_default=None)

    class Meta:
        strict = True
        ordered = True
        model = Images
        unknown = EXCLUDE

    @post_load
    def make_images(self, data, **kwargs):
        return Images(**data)



from soccer_sdk_utils.model.meta import MetaProperty


class Club:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.city = kwargs.get("city")
        self.state = kwargs.get("state")
        self.meta = kwargs.get("meta", [])

    @property
    def league(self):
        return self.get_property("league")

    def __repr__(self):
        return f"<Club(id={self.id}, name='{self.name}', location='{self.city}, {self.state}')>"

    def has_property(self, name: str):
        for meta in self.meta:
            if meta.name == name:
                return True

        return False

    def get_property(self, name: str):
        for meta in self.meta:
            if meta.name == name:
                return meta.value

        return None

    def add_property(self, name: str, value: str):
        self.meta.append(MetaProperty(name=name, value=value))


class ClubCommitmentStats:
    def __init__(self, **kwargs):
        self.club = kwargs.get("club")
        self.di = kwargs.get("di")
        self.dii = kwargs.get("dii")
        self.diii = kwargs.get("diii")
        self.naia = kwargs.get("naia")
        self.total = kwargs.get("total")

    def __repr__(self):
        buffer = "<ClubCommitmentStats("
        buffer += f"club={self.club}, "
        buffer += f"di={self.di}, "
        buffer += f"dii={self.dii}, "
        buffer += f"diii={self.diii}, "
        buffer += f"naia={self.naia}, "
        buffer += f"total={self.total})>"

        return buffer



class ClubTranslation:
    def __init__(self, **kwargs):
        for arg in kwargs:
            value = kwargs.get(arg)

            if arg in ["_id", "id"]:
                self.id = value
            elif arg in ["src", "from"]:
                self.src = value
            elif arg in ["dst", "to"]:
                self.dst = value
            else:
                raise ValueError("Invalid argument: %s" % arg)

    @property
    def is_noop(self):
        return self.src == self.dst

    def __repr__(self):
        return f"<ClubTranslation(src='{self.src}', dst='{self.dst}', id='{self.id}')>"



class CoachingChange:
    def __init__(self, **kwargs):
        self.program = kwargs.get("program")
        self.program_url = kwargs.get("program_url")
        self.old_coach = kwargs.get("old_coach")
        self.old_coach_url = kwargs.get("old_coach_url")
        self.new_coach = kwargs.get("new_coach")
        self.new_coach_url = kwargs.get("new_coach_url")
        self.clgid = kwargs.get("clgid")

    def __repr__(self):
        buffer = "<CoachingChange("
        buffer += f"program='{self.program}', "
        buffer += f"program_url='{self.program_url}', "
        buffer += f"old_coach='{self.old_coach}', "
        buffer += f"old_coach_url='{self.old_coach_url}', "
        buffer += f"new_coach='{self.new_coach}', "
        buffer += f"new_coach_url='{self.new_coach_url}', "
        buffer += f"clgid='{self.clgid}')>"

        return buffer



class Image:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.filename = kwargs.get("filename")
        self.path = kwargs.get("path")
        self.title = kwargs.get("title")
        self.alt_text = kwargs.get("alt_text")
        self.url = kwargs.get("url")

    def __repr__(self):
        return f"<Image(id={self.id}, filename='{self.filename}', path='{self.path}'), url='{self.url}'>"



import uuid


class StateProvince:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.code = kwargs.get("code")
        self.country = kwargs.get("country")

        if self.id is None:
            self.id = str(uuid.uuid4())

    def __repr__(self):
        return f"<StateProvince(name='{self.name}' code='{self.code}', id='{self.id}', country='{self.country}')>"



from marshmallow import EXCLUDE, Schema, ValidationError, fields, post_load

from soccer_sdk_utils.model.country import Country


class CountrySchema(Schema):
    id = fields.UUID(required=True)
    name = fields.Str(required=True)
    code = fields.Str(required=True)

    class Meta:
        strict = True
        ordered = True
        model = Country
        unknown = EXCLUDE

    @post_load
    def make_country(self, data, **kwargs):
        return Country(**data)

    @staticmethod
    def filter(country_dict: dict | None) -> dict | None:
        if country_dict is None:
            return None

        schema = CountrySchema(unknown=EXCLUDE)

        country_instance = schema.load(country_dict)
        country_dict = schema.dump(country_instance)

        return country_dict


if __name__ == "__main__":
    import logging
    import os
    import sys

    from dotenv import load_dotenv

    dirname = os.path.dirname(__file__)
    basedir = os.path.abspath("../..")
    load_dotenv(os.path.join(basedir, ".env"))

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    logger = logging.getLogger(__name__)

    from soccer_sdk_utils.dao import CountryDAO

    country_data = CountryDAO().get_all()

    try:
        result = CountrySchema(unknown=EXCLUDE).load(country_data, many=True)

        for country in result:
            logger.info(country)
    except ValidationError as err:
        logger.error(err.messages)
        sys.exit(1)



from marshmallow import EXCLUDE, Schema, fields, post_load, validate

from soccer_sdk_utils.model.player import Player
from soccer_sdk_utils.schema.meta import MetaPropertySchema


class PlayerSchema(Schema):
    id = fields.UUID(required=True, allow_none=True)
    gender = fields.Str(required=True, validate=validate.OneOf(["Male", "Female"]))
    first_name = fields.Str(required=True, allow_none=True)
    last_name = fields.Str(required=True, allow_none=True)
    club = fields.Str(required=True, allow_none=True)
    state = fields.Str(required=True, allow_none=True)
    position = fields.Str(required=True, allow_none=True)

    meta = fields.List(fields.Nested(MetaPropertySchema))

    class Meta:
        strict = True
        ordered = True
        model = Player
        unknown = EXCLUDE

    @post_load
    def make_player(self, data, **kwargs):
        return Player(**data)


if __name__ == "__main__":
    import logging
    import os
    from pprint import pprint

    from dotenv import load_dotenv

    dirname = os.path.dirname(__file__)
    basedir = os.path.abspath("../..")
    load_dotenv(os.path.join(basedir, ".env"))

    from common.dao.player import PlayerDAO

    player_dao = PlayerDAO()

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )

    player = Player()

    player.first_name = "Fred"
    player.gender = "Male"
    player.last_name = "Flintstone"
    player.position = "Forward"
    player.city = "Bedrock"
    player.state = "NA"
    player.country = "Narnia"
    player.league = "ECNL"
    player.team = "U19 Bedrock FC"

    player.add_property("grad_year", "2023")

    player_data = PlayerSchema().dump(player)

    pprint(player_data)

    # first_name = "Alena"
    # last_name = "Watts"
    # player_data = player_dao.get_by_fname_lname(first_name, last_name)

    # if player_data is None:
    #     logging.error(f"Player '{first_name} {last_name}' not found!")
    #     sys.exit(1)
    #
    # try:
    #     result = PlayerSchema(unknown=EXCLUDE).load(player_data)
    #     logging.info(result)
    # except ValidationError as err:
    #     logging.error(err)

    # player_data = PlayerDAO().search("Female", "2024", "GA")
    #
    # if player_data is None:
    #     logging.error(f"Failed to search for players!")
    #     sys.exit(2)
    #
    # try:
    #     result = PlayerSchema(unknown=EXCLUDE).load(player_data, many=True)
    #
    #     for player in result:
    #         logging.info(player)
    # except ValidationError as err:
    #     logger.error(err)
    #     sys.exit(3)



import logging

from azure.cosmos import exceptions
from marshmallow import EXCLUDE

from soccer_sdk_utils.dao.base import DataAccessObject
from soccer_sdk_utils.dao.club_translation import ClubTranslationDAO
from soccer_sdk_utils.schema.club import ClubSchema


class ClubDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("club", "/id", **kwargs)

    def get_by_name(self, name: str):
        """
        Get a club by name

        :param name: Name of the club
        :return: Club data as a dict
        """
        logging.info(f"Fetching club with name {name}")

        name = ClubTranslationDAO().translate(name)

        try:
            query = f"{self._select_prefix()} c WHERE c.name = '{name}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            clubs = list(items)

            if len(clubs) == 0:
                return None
            else:
                return clubs[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err

    def lookup_league(self, club_name: str) -> str | None:
        club_data = self.get_by_name(club_name)

        if club_data is None:
            return None

        club = ClubSchema(unknown=EXCLUDE).load(club_data)

        if club is None:
            return None

        return club.league

    def get_by_league(self, league: str):
        logging.info(f"Fetching clubs in league {league}")

        try:
            query = f'SELECT c.name FROM {self.cname} c JOIN (SELECT VALUE t FROM t in c.leagues WHERE t.name = "{league}")'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return list(items)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err



from marshmallow import EXCLUDE, Schema, ValidationError, fields, post_load

from soccer_sdk_utils.model.club import Club, ClubCommitmentStats
from soccer_sdk_utils.schema.meta import MetaPropertySchema


class ClubSchema(Schema):
    id = fields.UUID(required=True)
    name = fields.Str(required=True)
    city = fields.Str(allow_none=True, load_default=None)
    state = fields.Str(required=True)
    meta = fields.List(fields.Nested(MetaPropertySchema))

    class Meta:
        strict = True
        ordered = True
        model = Club
        unknown = EXCLUDE

    @post_load
    def make_club(self, data, **kwargs):
        return Club(**data)

    @staticmethod
    def filter(club_dict: dict | None) -> dict | None:
        if club_dict is None:
            return None

        schema = ClubSchema(unknown=EXCLUDE)

        club_instance = schema.load(club_dict)
        club_dict = schema.dump(club_instance)

        return club_dict


class ClubCommitmentStatsSchema(Schema):
    club = fields.Str(required=True)
    di = fields.Int(required=True)
    dii = fields.Int(required=True)
    diii = fields.Int(required=True)
    naia = fields.Int(required=True)
    total = fields.Int(required=True)

    class Meta:
        strict = True
        ordered = True
        model = ClubCommitmentStats
        unknown = EXCLUDE

    @post_load
    def make_club_commitment_stats(self, data, **kwargs):
        return ClubCommitmentStats(**data)


if __name__ == "__main__":
    import logging
    import os
    import sys

    from dotenv import load_dotenv

    dirname = os.path.dirname(__file__)
    basedir = os.path.abspath("../..")
    load_dotenv(os.path.join(basedir, ".env"))

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    logger = logging.getLogger(__name__)

    from soccer_sdk_utils.dao.club import ClubDAO

    club_data = ClubDAO().get_all()

    try:
        result = ClubSchema(unknown=EXCLUDE).load(club_data, many=True)

        for club in result:
            logger.info(club)
    except ValidationError as err:
        logger.error(err.messages)
        sys.exit(1)



import uuid


class Values:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.sw = kwargs.get("sw")
        self.tds = kwargs.get("tds")
        self.ecnl = kwargs.get("ecnl")

    def generate_id(self):
        """Generates a unique ID for the user in place"""
        self.id = str(uuid.uuid4())



class Match:
    def __init__(self, **kwargs):
        self.date = kwargs.get("date")
        self.gender = kwargs.get("gender")
        self.division = kwargs.get("division")
        self.team1 = kwargs.get("team1")
        self.team2 = kwargs.get("team2")
        self.score1 = kwargs.get("score1", 0)
        self.score2 = kwargs.get("score2", 0)

    def __repr__(self):
        if self.date is None:
            if self.division is None:
                return f"{self.team1} [{self.score1}] - {self.team2} [{self.score2}]"
            else:
                return f"{self.division} {self.team1} [{self.score1}] - {self.team2} [{self.score2}]"
        else:
            if self.division is None:
                return f"{self.date} {self.team1} [{self.score1}] - {self.team2} [{self.score2}]"
            else:
                return f"{self.date} {self.division} {self.team1} [{self.score1}] - {self.team2} [{self.score2}]"

    @property
    def is_draw(self):
        return self.score1 == self.score2

    @property
    def winner(self):
        if self.is_draw:
            return None

        if self.score1 is None or self.score2 is None:
            return None

        if self.score1 > self.score2:
            return self.team1

        return self.team2

    @property
    def loser(self):
        if self.is_draw:
            return None

        if self.score1 is None or self.score2 is None:
            return None

        if self.score1 < self.score2:
            return self.team1

        return self.team2

    def has_team(self, team):
        return self.team1 == team or self.team2 == team

    def get_score(self, team):
        if self.team1 == team:
            return self.score1

        if self.team2 == team:
            return self.score2

        return None

    def opponent(self, team):
        if self.team1 == team:
            return self.team2

        if self.team2 == team:
            return self.team1

        return None



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class UserDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("user", "/id", **kwargs)

    def revoke_token(self, jti):
        logging.info(f"Revoking token with jti {jti}")

        try:
            query = f'{self._select_prefix()} c WHERE c.jti = "{jti}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            users = list(items)

            if len(users) == 0:
                return None

            user = users[0]
            user["revoked"] = True

            self.container.upsert_item(user)

            return user
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def email_exists(self, email: str):
        logging.info(f"Checking if email {email} exists")

        try:
            query = f'{self._select_prefix()} c WHERE c.email = "{email}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            users = list(items)

            if len(users) == 0:
                return False

            return True
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def username_exists(self, username: str):
        logging.info(f"Checking if username {username} exists")

        try:
            query = f'{self._select_prefix()} c WHERE c.username = "{username}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            users = list(items)

            if len(users) == 0:
                return False

            return True
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def delete_by_email(self, email: str):
        logging.info(f"Deleting user with email {email}")

        try:
            user_data = self.get_by_email(email)

            if user_data is None:
                return

            UserDAO().delete(user_data["id"])

            return
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_username(self, username: str):
        logging.info(f"Fetching user with username {username}")

        try:
            query = f'{self._select_prefix()} c WHERE c.username = "{username}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            users = list(items)

            if len(users) == 0:
                return None

            return users[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_email(self, email: str):
        logging.info(f"Fetching user with email {email}")

        try:
            query = f'{self._select_prefix()} c WHERE c.email = "{email}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            users = list(items)

            if len(users) == 0:
                return None

            return users[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.image import Image


class ImageSchema(Schema):
    id = fields.Str(required=True, allow_none=True, load_default=None)
    filename = fields.Str(required=True, allow_none=True, load_default=None)
    path = fields.Str(required=True, allow_none=True, load_default=None)
    title = fields.Str(required=True, allow_none=True, load_default=None)
    alt_text = fields.Str(required=True, allow_none=True, load_default=None)
    url = fields.Str(required=True, allow_none=True, load_default=None)

    class Meta:
        strict = True
        ordered = True
        model = Image
        unknown = EXCLUDE

    @post_load
    def make_image(self, data, **kwargs):
        return Image(**data)



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.match import Match


class MatchSchema(Schema):
    id = fields.UUID(required=True)
    gender = fields.Str(required=True)
    date = fields.Date(required=True)
    division = fields.Str(required=True)
    team1 = fields.Str(required=True)
    team2 = fields.Str(required=True)
    score1 = fields.Int(required=True)
    score2 = fields.Int(required=True)

    class Meta:
        strict = True
        ordered = True
        model = Match
        unknown = EXCLUDE

    @post_load
    def make_match(self, data, **kwargs):
        return Match(**data)



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.ncaa import School


class SchoolSchema(Schema):
    name = fields.Str(required=True)
    conference = fields.Str(required=True)
    private = fields.Bool(required=True)
    hbcu = fields.Bool(required=True)
    state = fields.Str(required=True)

    class Meta:
        strict = True
        ordered = True
        model = School
        unknown = EXCLUDE

    @post_load
    def make_school(self, data, **kwargs):
        return School(**data)



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.league import League


class LeagueSchema(Schema):
    name = fields.Str(allow_none=True, load_default=None)
    conference = fields.Str(allow_none=True, load_default=None)

    class Meta:
        strict = True
        ordered = True
        model = League
        unknown = EXCLUDE

    @post_load
    def make_league(self, data, **kwargs):
        return League(**data)



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.suggestion import Suggestion


class SuggestionSchema(Schema):
    id = fields.Str()
    description = fields.Str()
    author = fields.Str()
    state = fields.Str()
    opened_on = fields.DateTime()
    closed_on = fields.DateTime()

    class Meta:
        strict = True
        ordered = True
        model = Suggestion
        unknown = EXCLUDE

    @post_load
    def make_suggestion(self, data, **kwargs):
        return Suggestion(**data)


if __name__ == "__main__":
    pass



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class SuggestionDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("suggestion", "/id", **kwargs)

    def get_by_state(self, state):
        logging.info(f"Fetching suggestions with state {state}")

        try:
            query = f"{self._select_prefix()} c WHERE c.state = '{state}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return list(items)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



from soccer_sdk_utils.model.values import Values


class School:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.gender = kwargs.get("gender")
        self.ids = kwargs.get("ids")
        self.urls = kwargs.get("urls")

        if self.ids is None:
            self.ids = Values()

        if self.urls is None:
            self.urls = Values()

    def __repr__(self):
        if self.id is None:
            return f"<School(name='{self.name}', gender='{self.gender}')>"
        else:
            return f"<School(id={self.id}, name='{self.name}', gender='{self.gender}')>"



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class CountryDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("country", "/id", **kwargs)

    def exists(self, name: str) -> bool:
        logging.info(f"Checking if country exists with name {name}")

        try:
            query = f"{self._select_prefix()} c WHERE c.name = '{name}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            countries = list(items)

            return len(countries) > 0
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_name(self, name: str):
        logging.info(f"Fetching country with name {name}")

        try:
            query = f"{self._select_prefix()} c WHERE c.name = '{name}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            countries = list(items)

            if len(countries) == 0:
                return None
            else:
                return countries[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



import uuid


class City:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.spid = kwargs.get("spid")

        if self.id is None:
            self.id = str(uuid.uuid4())

    def __repr__(self):
        return f"<City(name='{self.name}' id='{self.id}', spid='{self.spid}')>"



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.school import School
from soccer_sdk_utils.schema.values import ValuesSchema


class SchoolSchema(Schema):
    id = fields.UUID(required=True)
    name = fields.Str(required=True)
    gender = fields.Str(required=True)
    ids = fields.Nested(ValuesSchema)
    urls = fields.Nested(ValuesSchema)

    class Meta:
        strict = True
        ordered = True
        model = School
        unknown = EXCLUDE

    @post_load
    def make_school(self, data, **kwargs):
        return School(**data)



from marshmallow import EXCLUDE, Schema, ValidationError, fields, post_load

from soccer_sdk_utils.model.conference import Conference
from soccer_sdk_utils.schema.values import ValuesSchema


class ConferenceSchema(Schema):
    id = fields.UUID()
    name = fields.Str(allow_none=True, load_default=None)
    gender = fields.Str(allow_none=True, load_default=None)
    division = fields.Str(allow_none=True, load_default=None)
    ids = fields.Nested(ValuesSchema)
    urls = fields.Nested(ValuesSchema)

    class Meta:
        strict = True
        ordered = True
        model = Conference
        unknown = EXCLUDE

    @post_load
    def make_conference(self, data, **kwargs):
        return Conference(**data)


if __name__ == "__main__":
    import logging
    import os
    import sys

    from dotenv import load_dotenv

    dirname = os.path.dirname(__file__)
    basedir = os.path.abspath("../..")
    load_dotenv(os.path.join(basedir, ".env"))

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    logger = logging.getLogger(__name__)

    from soccer_sdk_utils.dao import ConferenceDAO

    conference_data = ConferenceDAO().get_all()

    try:
        result = ConferenceSchema(unknown=EXCLUDE).load(conference_data, many=True)

        for conference in result:
            logger.info(conference)
    except ValidationError as err:
        logger.error(err.messages)
        sys.exit(1)



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.team import Team


class TeamSchema(Schema):
    id = fields.Str(allow_none=True, load_default=None)
    title = fields.Str(allow_none=True, load_default=None)
    abbreviation = fields.Str(allow_none=True, load_default=None)

    class Meta:
        strict = True
        ordered = True
        model = Team
        unknown = EXCLUDE

    @post_load
    def make_team(self, data, **kwargs):
        return Team(**data)



from soccer_sdk_utils.model.values import Values


class Commitment:
    def __init__(self, **kwargs):
        self.school = kwargs.get("school")
        self.urls = kwargs.get("urls")

    @property
    def school(self) -> str | None:
        return self._school

    @school.setter
    def school(self, value: str | None):
        self._school = value

    @property
    def urls(self) -> Values:
        return self._urls

    @urls.setter
    def urls(self, value: Values):
        self._urls = value



class Transfer:
    def __init__(self, **kwargs):
        self.name = kwargs.get("name")
        self.url = kwargs.get("url")
        self.position = kwargs.get("position")
        self.former_school_name = kwargs.get("former_school_name")
        self.former_school_url = kwargs.get("former_school_url")
        self.new_school_name = kwargs.get("new_school_name")
        self.new_school_url = kwargs.get("new_school_url")

    def __repr__(self):
        buffer = "<Transfer("
        buffer += f"name={self.name}, "
        buffer += f"url={self.url}, "
        buffer += f"position={self.position}, "
        buffer += f"former_school_name={self.former_school_name}, "
        buffer += f"former_school_url={self.former_school_url}, "
        buffer += f"new_school_name={self.new_school_name}, "
        buffer += f"new_school_url={self.new_school_url})"

        return buffer



class School:
    def __init__(self, **kwargs):
        self.name = kwargs.get("nameOfficial")
        self.conference = kwargs.get("conferenceName")
        self.private = kwargs.get("privateFlag") == "Y"
        self.hbcu = kwargs.get("historicallyBlackFlag") == "Y"
        self.state = kwargs.get("memberOrgAddress")["state"]

    def __repr__(self):
        buffer = "<School("
        buffer += f"name='{self.name}', "
        buffer += f"conference='{self.conference}', "
        buffer += f"private={self.private}, "
        buffer += f"hbcu={self.hbcu}, "
        buffer += f"state='{self.state}')>"

        return buffer

    def __str__(self):
        return self.__repr__()



import logging
import os
import sys
import uuid

from azure.cosmos import CosmosClient, PartitionKey, exceptions
from dotenv import load_dotenv


class DataAccessObject(object):
    def __init__(self, *args, **kwargs):
        self.cname = args[0]
        self.partition_key = args[1]

        self.last_query = None

        debug = kwargs.get("debug", False)

        basedir = sys.path[0]
        load_dotenv(os.path.join(basedir, ".env"))

        db_name = os.environ.get("DATABASE_NAME")

        if hasattr(sys, "_called_from_test"):
            db_name += "-test"

        acct_uri = os.environ["COSMOS_ACCOUNT_URI"]
        acct_key = os.environ["COSMOS_ACCOUNT_KEY"]

        if debug:
            # Create a logger for the 'azure' SDK
            logger = logging.getLogger("azure")
            logger.setLevel(logging.DEBUG)

            # Configure a console output
            handler = logging.StreamHandler(stream=sys.stdout)
            logger.addHandler(handler)

            self.client = CosmosClient(
                url=acct_uri, credential=acct_key, logging_enable=True
            )
        else:
            self.client = CosmosClient(url=acct_uri, credential=acct_key)

        try:
            # Try to create the database if it doesn't exist
            self.db = self.client.create_database(db_name)
        except exceptions.CosmosResourceExistsError:
            self.db = self.client.get_database_client(db_name)

        try:
            self.container = self.db.create_container(
                self.cname, PartitionKey(path=self.partition_key)
            )
        except exceptions.CosmosResourceExistsError:
            self.container = self.db.get_container_client(self.cname)

    def _select_prefix(self):
        return f"SELECT * FROM {self.cname}"

    def get_all(self):
        logging.info(f"Fetching all {self.cname}s")

        try:
            query = f"{self._select_prefix()} c"
            self.last_query = query
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return items
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)
            raise err
        except Exception as err:
            logging.error(err)
            raise err

    def get(self, id):
        logging.info(f"Fetching {self.cname} {id}")
        try:
            query = f"{self._select_prefix()} c WHERE c.id = '{id}'"
            self.last_query = query
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            for item in items:
                if item.get("id") != id:
                    continue

                return item

            return None
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def create(self, data):
        logging.info(f"Creating {self.cname}")

        try:
            if not data.get("id"):
                data["id"] = str(uuid.uuid4())

            return self.container.create_item(data)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def update(self, id, data):
        logging.info(f"Updating {self.cname} {id}")

        try:
            return self.container.upsert_item(data, id)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def delete(self, id):
        logging.info(f"Deleting {self.cname} {id}")

        try:
            query = f"{self._select_prefix()} c WHERE c.id = '{id}'"
            self.last_query = query
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )
            for item in items:
                self.container.delete_item(item, id)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.values import Values


class UrlsSchema(Schema):
    sw = fields.URL(allow_none=True, load_default=None)
    tds = fields.URL(allow_none=True, load_default=None)

    class Meta:
        strict = True
        ordered = True
        model = Values
        unknown = EXCLUDE

    @post_load
    def make_urls(self, data, **kwargs):
        return Values(**data)



class Country:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.code = kwargs.get("code")

    def __repr__(self):
        return f"<Country(id={self.id}, name='{self.name}', code='{self.code}')>"



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.transfer import Transfer


class TransferSchema(Schema):
    name = fields.Str(allow_none=False, load_default="")
    url = fields.Str(allow_none=True, load_default=None)
    position = fields.Str(allow_none=True, load_default=None)
    former_school_name = fields.Str(allow_none=True, load_default=None)
    former_school_url = fields.Str(allow_none=True, load_default=None)
    new_school_name = fields.Str(allow_none=True, load_default=None)
    new_school_url = fields.Str(allow_none=True, load_default=None)

    class Meta:
        strict = True
        ordered = True
        model = Transfer
        unknown = EXCLUDE

    @post_load
    def make_transfer(self, data, **kwargs):
        return Transfer(**data)



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class ImageDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("image", "/id", **kwargs)

    def get_by_url(self, url: str):
        logging.info(f"Fetching image with url {url}")

        try:
            query = f"{self._select_prefix()} i WHERE i.url = '{url}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            images = list(items)

            if len(images) == 0:
                return None
            else:
                return images[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



import datetime as dt
import uuid

from flask import current_app
from flask_bcrypt import check_password_hash, generate_password_hash

from soccer_sdk_utils.model.meta import MetaProperty


class User:
    def __init__(self, **kwargs):
        name = kwargs.get("name")

        if name is not None:
            tokens = name.split(" ")
            self.first_name = tokens[0].strip()
            self.last_name = tokens[1].strip()
        else:
            self.first_name = kwargs.get("first_name")
            self.last_name = kwargs.get("last_name")

            if self.first_name is not None:
                self.first_name = self.first_name.strip()

            if self.last_name is not None:
                self.last_name = self.last_name.strip()

        self.id = kwargs.get("id")
        self.email = kwargs.get("email")
        self.username = kwargs.get("username")
        self.password = kwargs.get("password")
        self.phone = kwargs.get("phone")
        self.registered_on = kwargs.get("registered_on")
        self.meta = kwargs.get("meta", [])

    @property
    def slug(self):
        return f"{self.first_name}-{self.last_name}".lower()

    def check_password(self, password: str) -> bool:
        return check_password_hash(self.password, password)

    def hash_password(self):
        """Hashes the user's password in place"""
        self.password = generate_password_hash(self.password).decode("utf-8")

    def generate_id(self):
        """Generates a unique ID for the user in place"""
        self.id = str(uuid.uuid4())

    def has_property(self, name: str):
        for meta in self.meta:
            if meta.name == name:
                return True

        return False

    def get_property(self, name: str):
        for meta in self.meta:
            if meta.name == name:
                return meta.value

        return None

    def add_property(self, name: str, value: str):
        self.meta.append(MetaProperty(name=name, value=value))

    @staticmethod
    def get_utc_now():
        return dt.datetime.utcnow()

    @staticmethod
    def get_secret_key():
        return current_app.config.get("SECRET_KEY")

    def __repr__(self):
        return f"<User(email='{self.email}', first_name='{self.first_name}', last_name='{self.last_name}')>"



from marshmallow import EXCLUDE, Schema, ValidationError, fields, post_load

from soccer_sdk_utils.model.club_translation import ClubTranslation


class ClubTranslationSchema(Schema):
    id = fields.Str(required=True)
    src = fields.Str(required=True, data_key="from")
    dst = fields.Str(required=True, data_key="to")

    class Meta:
        strict = True
        ordered = True
        model = ClubTranslation
        unknown = EXCLUDE

    @post_load
    def make_translation(self, data, **kwargs):
        return ClubTranslation(**data)

    @staticmethod
    def filter(translation_dict: dict | None) -> dict | None:
        if translation_dict is None:
            return None

        schema = ClubTranslationSchema(unknown=EXCLUDE)
        instance = schema.load(translation_dict)
        translation_dict = schema.dump(instance)

        return translation_dict


if __name__ == "__main__":
    import logging
    import os
    import sys

    from dotenv import load_dotenv

    dirname = os.path.dirname(__file__)
    basedir = os.path.abspath("../..")
    load_dotenv(os.path.join(basedir, ".env"))

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    logger = logging.getLogger(__name__)

    from soccer_sdk_utils.dao.club_translation import ClubTranslationDAO
    from soccer_sdk_utils.model.club_translation import ClubTranslation

    data = ClubTranslationDAO().get_all()

    try:
        results = ClubTranslationSchema(unknown=EXCLUDE).load(data, many=True)

        for result in results:
            logging.info(result)
    except ValidationError as err:
        logging.error(err)
        sys.exit(1)



class MetaProperty:
    def __init__(self, **kwargs):
        self.name = kwargs.get("name")
        self.value = kwargs.get("value")

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, input_name: str):
        if input_name is not None:
            input_name = input_name.strip()

        self._name = input_name

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, input_value: str):
        if input_value is not None:
            if type(input_value) != str:
                input_value = str(input_value)

            input_value = input_value.strip()

        self._value = input_value

    def __repr__(self):
        if self.value is None:
            return f"{self.name}=null"
        else:
            return f"{self.name}='{self.value}'"



class BlacklistToken:
    """
    This method is used to save a token to the blacklist table
    """

    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.token = kwargs.get("token")
        self.blacklisted_on = kwargs.get("blacklisted_on")

    def __repr__(self):
        return f"BlacklistToken(id={self.id}, token={self.token}, blacklisted_on={self.blacklisted_on})"



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject
from soccer_sdk_utils.dao.country import CountryDAO


class StateProvinceError(Exception):
    def __init__(self, message):
        super().__init__(message)


class StateProvinceDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("state_province", "/id", **kwargs)

    def exists(self, name: str):
        logging.info(f"Checking if state/province with name {name} exists")

        try:
            return self.get_by_name(name) is not None
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_name(self, name: str):
        logging.info(f"Fetching state/province with name {name}")

        try:
            query = f"{self._select_prefix()} c WHERE c.name = '{name}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            states_provinces = list(items)

            if len(states_provinces) == 0:
                return None
            else:
                return states_provinces[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_code(self, code: str):
        logging.info(f"Fetching state/province with code {code}")

        try:
            query = f"{self._select_prefix()} c WHERE c.code = '{code}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            states_provinces = list(items)

            if len(states_provinces) == 0:
                return None
            else:
                return states_provinces[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_country_name(self, name: str):
        logging.info(f"Fetching state/province with country name {name}")

        try:
            country_dao = CountryDAO()
            country = country_dao.get_by_name(name)

            if country is None:
                raise StateProvinceError(f"There are no countries with name {name}!")

            return self.get_by_country_id(country.get_soup("id"))
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_country_id(self, id: str):
        logging.info(f"Fetching state/province with country id {id}")

        try:
            query = f"{self._select_prefix()} c WHERE c.country = '{id}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return items
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



from marshmallow import EXCLUDE, Schema, ValidationError, fields, post_load

from soccer_sdk_utils.model.city import City


class CitySchema(Schema):
    id = fields.UUID(required=True)
    name = fields.Str(required=True)
    spid = fields.Str(required=True)

    class Meta:
        strict = True
        ordered = True
        model = City
        unknown = EXCLUDE

    @post_load
    def make_city(self, data, **kwargs):
        return City(**data)


if __name__ == "__main__":
    import logging
    import os
    import sys

    from dotenv import load_dotenv

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )

    dirname = os.path.dirname(__file__)
    basedir = os.path.abspath("../..")
    load_dotenv(os.path.join(basedir, ".env"))

    from common.dao import CityDAO

    city_data = CityDAO().get_all()

    try:
        result = CitySchema(unknown=EXCLUDE).load(city_data, many=True)

        for city in result:
            logging.info(city)
    except ValidationError as err:
        logging.error(err.messages)
        sys.exit(1)



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.meta import MetaProperty


class MetaPropertySchema(Schema):
    """MetaProperty schema."""

    class Meta:
        strict = True
        ordered = True
        model = MetaProperty
        unknown = EXCLUDE

    # pylint: disable=invalid-name
    name = fields.Str(required=True)
    value = fields.Str(required=True)

    @post_load
    def make_meta_property(self, data, **kwargs):
        """Make meta property."""
        return MetaProperty(**data)



from soccer_sdk_utils.gender import Gender
from soccer_sdk_utils.model.meta import MetaProperty


class Player:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.gender = kwargs.get("gender")
        self.first_name = kwargs.get("first_name")
        self.last_name = kwargs.get("last_name")
        self.club = kwargs.get("club")
        self.state = kwargs.get("state")
        self.position = kwargs.get("position")

        self.meta = kwargs.get("meta", [])

    @property
    def name(self) -> str:
        return f"{self.first_name} {self.last_name}"

    @name.setter
    def name(self, value: str):
        if value is not None:
            parts = value.split(" ")

            if len(parts) == 2:
                self.first_name = parts[0]
                self.last_name = parts[1]
            elif len(parts) == 3:
                self.first_name = parts[0]
                self.last_name = f"{parts[1]} {parts[2]}"
            else:
                self.first_name = value
                self.last_name = None

    @property
    def year(self) -> str | None:
        result = self.get_property("grad_year")
        if result is None:
            result = self.get_property("graduation_year")

        return result

    @property
    def commitment(self) -> str | None:
        result = self.get_property("commitment")

        if result is None:
            result = self.get_property("college_team")

            if result is not None:
                if result.endswith(" Women"):
                    return result[:-6].strip()
                elif result.endswith(" Men"):
                    return result[:-4].strip()

        return result

    @property
    def is_committed(self) -> bool:
        commitment = self.commitment
        return commitment is not None and commitment != ""

    def __repr__(self):
        avps = []

        if self.id is not None:
            avps.append(f"id='{self.id}'")

        avps.append(f"name='{self.name}'")

        if self.year is not None:
            avps.append(f"year='{self.year}'")

        gender_type = type(self.gender)
        if gender_type == Gender:
            avps.append(f"gender='{self.gender.name}'")
        elif gender_type == str:
            avps.append(f"gender='{self.gender}'")

        if self.state is not None:
            avps.append(f"state='{self.state}'")

        avps.append(f"position='{self.position}'")

        if self.club is not None:
            avps.append(f"club='{self.club}'")

        if self.is_committed:
            avps.append(f"commitment='{self.commitment}'")

        return f"<Player({', '.join(avps)})>"

    def has_property(self, name: str) -> bool:
        for item in self.meta:
            if item.name == name:
                return item.value is not None and item.value != ""

        return False

    def get_property(self, name: str) -> str | None:
        for item in self.meta:
            if item.name == name:
                return item.value

        return None

    def update_property(self, name: str, value: str | None):
        for item in self.meta:
            if item.name == name:
                item.value = value

    def add_property(self, name: str, value: str | None):
        if value is not None:

            if type(value) == str:
                value = value.strip()

                if len(value) == 0:
                    value = None

        if self.has_property(name):
            self.update_property(name, value)
        else:
            self.meta.append(MetaProperty(name=name, value=value))



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class ClubTranslationDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("club_translation", "/id", **kwargs)

    def get_by_source(self, source: str) -> list[dict]:
        query = f'{self._select_prefix()} c WHERE c["from"] = "{source}"'
        try:
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            translations = list(items)

            return translations
        except exceptions.CosmosHttpResponseError as err:
            print(f"Query: {query}")
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_mapping(self, from_: str, to_: str):
        query = f'{self._select_prefix()} c WHERE c["from"] = "{from_}" AND c["to"] = "{to_}"'
        try:
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            translations = list(items)

            if len(translations) > 0:
                return translations[0]

            return None
        except exceptions.CosmosHttpResponseError as err:
            print(f"Query: {query}")
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def translate(self, name: str):
        if name is None:
            return None

        name = name.strip()

        query = f'{self._select_prefix()} c WHERE c["from"] = "{name}"'
        try:
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            translations = list(items)

            if len(translations) > 0:
                logging.info(f"Translating club name {name}")
                return translations[0]["to"]

            logging.info(f"No translation found for club name {name}")
            return name
        except exceptions.CosmosHttpResponseError as err:
            print(f"Query: {query}")
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.coach import CoachingChange


class CoachingChangeSchema(Schema):
    program = fields.Str(required=True)
    program_url = fields.Str(required=True)
    old_coach = fields.Str(required=False)
    old_coach_url = fields.Str(required=False)
    new_coach = fields.Str(required=False)
    new_coach_url = fields.Str(required=False)
    clgid = fields.Str(required=True)

    class Meta:
        strict = True
        ordered = True
        model = CoachingChange
        unknown = EXCLUDE

    @post_load
    def make_coaching_change(self, data, **kwargs):
        return CoachingChange(**data)



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.commitment import Commitment
from soccer_sdk_utils.schema.values import ValuesSchema


class CommitmentSchema(Schema):
    school = fields.Str(allow_none=True, load_default=None)
    urls = fields.Nested(ValuesSchema)

    class Meta:
        strict = True
        ordered = True
        model = Commitment
        unknown = EXCLUDE

    @post_load
    def make_commitment(self, data, **kwargs):
        return Commitment(**data)



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class BlacklistTokenDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("blacklist_token", "/id", **kwargs)

    def is_blacklisted(self, auth_token: str) -> bool:
        logging.info(f"Checking if token {auth_token} is blacklisted")

        try:
            query = f'{self._select_prefix()} c WHERE c.token = "{auth_token}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            tokens = list(items)

            if len(tokens) == 0:
                return False

            return True
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def delete_by_token(self, token: str):
        logging.info(f"Deleting token {token} from blacklist")

        try:
            query = f'{self._select_prefix()} c WHERE c.token = "{token}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            tokens = list(items)

            if len(tokens) == 0:
                return

            for token in tokens:
                self.delete(token.get("id"))
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



import uuid

from soccer_sdk_utils.model.values import Values
from soccer_sdk_utils.tools import slugify


class Conference:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.gender = kwargs.get("gender")
        self.division = kwargs.get("division")
        self.ids = kwargs.get("ids")
        self.urls = kwargs.get("urls")

        if self.ids is None:
            self.ids = Values()

        if self.urls is None:
            self.urls = Values()

    @property
    def slug(self) -> str:
        """Returns a slug for the conference"""
        return slugify(self.name)

    def __repr__(self):
        if self.id is None:
            return f"<Conference(name='{self.name}', division='{self.division}', gender='{self.gender.name}')>"
        else:
            return f"<Conference(id={self.id}, name='{self.name}', division='{self.division}', gender='{self.gender.name}')>"

    def generate_id(self):
        """Generates a unique ID for the user in place"""
        self.id = str(uuid.uuid4())



from datetime import datetime


class Suggestion:
    def __init__(
        self,
        _id: str = None,
        description: str = None,
        author: str = None,
        state: str = None,
        opened_on: str = None,
        closed_on: str = None,
    ):
        self.id = _id
        self.description = description
        self.author = author
        self.state = state
        self.opened_on = opened_on
        self.closed_on = closed_on

    def __repr__(self):
        buffer = "<Suggestion("
        buffer += f"id={self.id}, "
        buffer += f"description={self.description}, "
        buffer += f"author={self.author}, "
        buffer += f"state={self.state}, "
        buffer += f"opened_on={self.opened_on}, "
        buffer += f"closed_on={self.closed_on})>"

        return buffer

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, value: str):
        self._id = value

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value: str):
        self._description = value

    @property
    def author(self):
        return self._author

    @author.setter
    def author(self, value: str):
        self._author = value

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, value: str):
        self._state = value

    @property
    def opened_on(self):
        return self._opened_on

    @opened_on.setter
    def opened_on(self, value: datetime):
        self._opened_on = value

    @property
    def closed_on(self) -> datetime:
        return self._closed_on

    @closed_on.setter
    def closed_on(self, value: datetime):
        self._closed_on = value



from marshmallow import EXCLUDE, Schema, fields, post_load

from soccer_sdk_utils.model.bltoken import BlacklistToken


class BlacklistTokenSchema(Schema):
    id = fields.UUID(required=False)
    token = fields.Str(required=True)
    blacklisted_on = fields.DateTime(required=False, allow_none=True, load_default=None)

    class Meta:
        strict = True
        ordered = True
        model = BlacklistToken
        unknown = EXCLUDE

    @post_load
    def make_blacklist_token(self, data, **kwargs):
        return BlacklistToken(**data)

    @staticmethod
    def filter(blacklist_token_dict: dict | None) -> dict | None:
        if blacklist_token_dict is None:
            return None

        schema = BlacklistTokenSchema(unknown=EXCLUDE)

        blacklist_token_instance = schema.load(blacklist_token_dict)
        blacklist_token_dict = schema.dump(blacklist_token_instance)

        return blacklist_token_dict



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class CityDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("city", "/id", **kwargs)

    def get_by_state_province_id(self, id: str):
        logging.info(f"Fetching cities with state/province id {id}")

        try:
            query = f"{self._select_prefix()} c WHERE c.spid = '{id}'"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return items
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_name(self, name: str):
        logging.info(f"Fetching city with name {name}")

        try:
            query = f'{self._select_prefix()} c WHERE c.name = "{name}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            cities = list(items)

            if len(cities) == 0:
                return None

            return cities[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject


class PlayerDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("player", "/id", **kwargs)

    def search(self, gender: str, year: str, state: str):
        logging.info("Fetching players")

        try:
            query = f'{self._select_prefix()} c WHERE c.year = "{year}" AND c.gender = "{gender}" AND c.state = "{state}"'

            logging.info(f"Query: {query}")

            query_iterable = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return list(query_iterable)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_fname_lname(self, fname: str, lname: str):
        logging.info(f"Fetching player with first name {fname} and last name {lname}")

        try:
            query = f'{self._select_prefix()} c WHERE c.first_name = "{fname}" and c.last_name = "{lname}"'
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            players = list(items)

            if len(players) == 0:
                return None
            else:
                return players[0]
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



class League:
    def __init__(self, **kwargs):
        self.name = kwargs.get("name")
        self.conference = kwargs.get("conference")

    def __repr__(self):
        return f"<League(name='{self.name}', conference='{self.conference}')>"



import logging

from azure.cosmos import exceptions

from soccer_sdk_utils.dao.base import DataAccessObject
from soccer_sdk_utils.gender import Gender


class MatchDAO(DataAccessObject):
    def __init__(self, **kwargs):
        super().__init__("match", "/id", **kwargs)

    def find(self, gender: Gender, date, team1, team2, division=None):
        logging.info(f"Fetching match '{date} {team1} vs {team2}' {gender.name}")

        try:
            matches = self.get_by_date(gender, date, division)

            for match in matches:
                if match["team1"] == team1 and match["team2"] == team2:
                    return match

                if match["team1"] == team2 and match["team2"] == team1:
                    return match

            return None
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def exists(self, gender: Gender, date, team1, team2):
        logging.info(
            f"Checking if match '{date} {team1} vs {team2}' {gender.name} exists"
        )

        try:
            for match in self.get_by_date(gender, date):
                if match["team1"] == team1 and match["team2"] == team2:
                    return True

                if match["team1"] == team2 and match["team2"] == team1:
                    return True

            return False
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_date(self, gender: Gender, date, division=None):
        logging.info(
            f"Fetching matches on date '{date}' for division '{division}' {gender.name}"
        )

        try:
            query = f"{self._select_prefix()} c WHERE c.date = '{date}' AND "
            if division is None:
                query += f"c.gender = '{gender}'"
            else:
                query += f"c.division = '{division}' AND c.gender = '{gender}'"

            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return list(items)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_date_range(self, gender: Gender, date_start, date_end, division=None):
        logging.info(
            f"Fetching matches between '{date_start}' and '{date_end}' for division '{division}'"
        )

        try:
            query = f"{self._select_prefix()} c WHERE "
            query += f"c.gender = '{gender}' AND "
            if division is None:
                query += f"(c.date BETWEEN '{date_start}' AND '{date_end}')"
            else:
                query += f"c.division = '{division}' AND "
                query += f"(c.date BETWEEN '{date_start}' AND '{date_end}')"

            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return list(items)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err

    def get_by_team(self, gender: Gender, team):
        logging.info(f"Fetching matches for team '{team}' {gender.name}")

        try:
            query = f"{self._select_prefix()} c WHERE c.gender = '{gender}' AND (c.team1 = '{team}' OR c.team2 = '{team}')"
            items = self.container.query_items(
                query=query, enable_cross_partition_query=True
            )

            return list(items)
        except exceptions.CosmosHttpResponseError as err:
            logging.error(err)

            raise err
        except Exception as err:
            logging.error(err)

            raise err



import logging

from marshmallow import EXCLUDE, Schema, ValidationError, fields, post_load

from soccer_sdk_utils.model.user import User


class UserSchema(Schema):
    id = fields.UUID(required=False)
    first_name = fields.Str(required=False, allow_none=True, load_default=None)
    last_name = fields.Str(required=False, allow_none=True, load_default=None)
    username = fields.Str(required=False, allow_none=True, load_default=None)
    email = fields.Email(required=True)
    password = fields.Str(required=True)
    registered_on = fields.DateTime(required=False, allow_none=True, load_default=None)
    phone = fields.Str(required=False, allow_none=True, load_default=None)

    class Meta:
        strict = True
        ordered = True
        model = User
        unknown = EXCLUDE

    @post_load
    def make_user(self, data, **kwargs):
        return User(**data)

    @staticmethod
    def filter(user_dict: dict | None) -> dict | None:
        if user_dict is None:
            return None

        schema = UserSchema(unknown=EXCLUDE)

        user_instance = schema.load(user_dict)
        user_dict = schema.dump(user_instance)

        return user_dict


if __name__ == "__main__":
    import os

    from dotenv import load_dotenv

    dirname = os.path.dirname(__file__)
    basedir = os.path.abspath("../..")
    load_dotenv(os.path.join(basedir, ".env"))

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    logger = logging.getLogger(__name__)

    # Creating Schemas from Dictionaries
    # https://marshmallow.readthedocs.io/en/stable/quickstart.html#creating-schemas-from-dictionaries

    # Serializing Objects ("Dumping")
    logger.info("Serializing Objects ('Dumping')")
    user = User(password="something", email="john.doe@python.org")
    schema = UserSchema(unknown=EXCLUDE)
    result = schema.dump(user)
    logger.info(result)

    # Filtering Output
    logger.info("Filtering Output")
    summary_schema = UserSchema(unknown=EXCLUDE, only=("id", "email"))
    result = summary_schema.dump(user)
    logger.info(result)

    # Deserializing Objects ("Loading")
    logger.info("Deserializing Objects ('Loading')")
    # https://marshmallow.readthedocs.io/en/stable/quickstart.html#deserializing-objects-loading
    user_data = {
        "id": "d15dc781-4780-4994-b9ec-5771376c2a2e",
        "email": "janedoe@yahoo.com",
        "password": "something",
    }

    schema = UserSchema(unknown=EXCLUDE)
    result = schema.load(user_data)
    logger.info(result)

    # Handling Collections of Objects
    logger.info("Handling Collections of Objects")
    user1 = User(email="mick@stones.com")
    user2 = User(email="keith@stones.com")
    users = [user1, user2]
    schema = UserSchema(unknown=EXCLUDE, many=True)
    result = schema.dump(users)  # Or UserSchema().dump(users, many=True)
    logger.info(result)

    user_data = [
        {
            "id": "d15dc781-4780-4994-b9ec-5771376c2a2e",
            "email": "janedoe@yahoo.com",
            "password": "something else",
        },
        {
            "id": "d15dc781-4780-4994-b9ec-5771376c2a2f",
            "email": "jonedoe@gmail.com",
            "password": "something",
        },
    ]

    result = UserSchema(unknown=EXCLUDE).load(user_data, many=True)
    logger.info(result)

    from common.dao import UserDAO

    logging.info("Loading data from the database ...")
    user_data = UserDAO().get_all()

    try:
        users = UserSchema(unknown=EXCLUDE).load(user_data, many=True)

        for user in users:
            logger.info(user)
    except ValidationError as err:
        logger.error(err.messages)
        logger.error(err.valid_data)
